#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Mar 26 09:56:34 2019

@author: stan
"""

from abc import ABC,abstractmethod

class ExpresionBooleana(ABC):
    
    @abstractmethod
    def evaluate(self,assignment):
        pass
    
    def __repr__(self):
        return str(self)
    
class TRUE(ExpresionBooleana):
    
    def evaluate(self,assignment):
        return True
    
    def __str__(self):
        return "1"
    
class FALSE(ExpresionBooleana):
    
    def evaluate(false,assignment):
        return False
    
    def __str__(self):
        return "0"

class Var(ExpresionBooleana):
    
    def __init__(self,nombre):
        self.nombre = nombre
        
    def evaluate(self,assignment):
        return assignment[self.nombre]
        
    def __str__(self):
        return self.nombre
    
    
class Not(ExpresionBooleana):
    
    def __init__(self, expresion):
        self.param = expresion
        
    def evaluate(self,assignment):
        return not self.param.evaluate(assignment)
        
    def get_symbol(self):
        return "~"
        
    def __str__(self):
        return self.get_symbol()+str(self.param)
    
class ExpresionBinaria(ExpresionBooleana,ABC):

    def __init__(self,left,right):
        self.left = left
        self.right = right
    
    @abstractmethod
    def get_symbol(self):
        pass
    
    def __str__(self):
        return "("+str(self.left)+self.get_symbol()+str(self.right)+")"
    
class And(ExpresionBinaria):
    
    def __init__(self,left,right):
        super().__init__(left,right)
        
    def evaluate(self,assignment):
        return  self.left.evaluate(assignment) and\
                self.right.evaluate(assignment)
        
    def get_symbol(self):
        return "&"
    
class Or(ExpresionBinaria):
    
    def __init__(self,left,right):
        super().__init__(left,right)
        
    def evaluate(self,assignment):
        return  self.left.evaluate(assignment) or\
                self.right.evaluate(assignment)
        
    def get_symbol(self):
        return "|"
    
class Implication(ExpresionBinaria):
    
    def __init__(self,left,right):
        super().__init__(left,right)
        
    def evaluate(self,assignment):
        return  not self.left.evaluate(assignment) or\
                self.right.evaluate(assignment)
        
    def get_symbol(self):
        return "->"
    
class Equivalence(ExpresionBinaria):
    
    def __init__(self,left,right):
        super().__init__(left,right)
    
    def evaluate(self,assignment):
        return  self.left.evaluate(assignment) ==\
                self.right.evaluate(assignment)
    
    def get_symbol(self):
        return "="
    
